/* $Id: sci_js.c,v 1.5 2002/04/30 13:41:29 dirk Exp $
 * Copyright (C) 2002 by Dirk Reusch <reusch@tmb.uka.de>
 */

#include <jsw.h>		/* libjsw */

#include "../../../routines/machine.h"

#define MAXDEVS 4

extern void sciprint ();

static int init = 0;

struct sci_js_struct
{
	js_data_struct jsd;	/* joystick data */
	int ref;		/* ref counter   */
};

static struct sci_js_struct sci_js[MAXDEVS];

static int
sci_js_init (int num)
{
	char device[10];

	int i, status;

	/* initialize module ? */
	if (init == 0)
	{
		for (i = 0; i < MAXDEVS; i++)
		{
			/* sci_js[i].jsd=-1; */
			sci_js[i].ref = 0;
		}
		init = 1;
	};
	
	/* 0 <= num < MAXDEVS ? */
	if ((num<0)||(num>=MAXDEVS))
	{
		sciprint("JOYSTICK: dev number %i out of range.\r\n", num);
		return -1;
	}

	/* initialize joystick ? */
	if (sci_js[num].ref <= 0)
	{
		/* initialize /dev/js<num> */
		snprintf (device, 10, "/dev/js%i", num);
		status = JSInit (&(sci_js[num].jsd),
				 device,
				 JSDefaultCalibration, JSFlagNonBlocking);
		if (status != JSSuccess)
		{
			sciprint ("JOYSTICK: %s: error %i.\r\n", device, status);
			JSClose (&(sci_js[num].jsd));
			return -1;
		}
	}

	/* increment ref counter */
	sci_js[num].ref++;
	
	sciprint("JOYSTICK: /dev/js%i: init (ref=%i).\r\n",
		num, sci_js[num].ref);
	
	return 0;
};

static void
sci_js_close (int num)
{
	if ((num>=0)&&(num<MAXDEVS))
	{
		if (sci_js[num].ref>0)
		{
			/* decrement ref counter */
			sci_js[num].ref--;

			/* close joystick device */
			if (sci_js[num].ref <= 0)
				JSClose (&(sci_js[num].jsd));
		}
		sciprint("JOYSTICK: /dev/js%i: close (ref=%i).\r\n",
			num, sci_js[num].ref);
	}
};

/*
 * SciCOS computational interface functions
 */

/*
 * JS_AXES
 */

int
sci_js_axes (flag, nevprt, t, xd, x, nx, z, nz, tvec, ntvec, rpar,
	       nrpar, ipar, nipar, inptr, insz, nin, outptr, outsz, nout)
     integer *flag, *nevprt, *nx, *nz, *ntvec, *nrpar, ipar[], *nipar;
     integer insz[], *nin, outsz[], *nout;
     double x[], xd[], z[], tvec[], rpar[];
     double *inptr[], *outptr[], *t;

/* Integer parameters
 *
 * ipar[0]   : device number
 * ipar[1:n] : list of axes
 *
 */

{
	if (*flag == 1)		/* outputs update */
	{
		int i;
		
		JSUpdate (&(sci_js[ipar[0]].jsd));

		/* retrieve axis values */
		for (i = 0; i < outsz[0]; i++)
			*(outptr[0] + i) =
				JSGetAxisCoeffNZ (&(sci_js[ipar[0]].jsd),
						  ipar[1 + i]);
	}
	else if (*flag == 4)	/* initialize */
	{
		int i, na;

		/* initialize joystick device */
		if (sci_js_init (ipar[0]) < 0)
		{
			*flag = -3;
			return 0;
		}

		/* number of requested axess */
		na = (*nipar - 1);

		/* check for requested axess */
		for (i = 1; i <= na; i++)
			if (!JSIsAxisAllocated
			    (&(sci_js[ipar[0]].jsd), ipar[i]))
			{
				sciprint ("JOYSTICK: /dev/js%i: axis %i not available.\r\n",
					  ipar[0], ipar[i]);
				*flag = -3;
				return 0;
			}
	}
	else if (*flag == 5)
		sci_js_close (ipar[0]);
	
	return 0;
};

/*
 * JS_BUTTONS
 */

int
sci_js_buttons (flag, nevprt, t, xd, x, nx, z, nz, tvec, ntvec, rpar,
		nrpar, ipar, nipar, inptr, insz, nin, outptr, outsz, nout)
     integer *flag, *nevprt, *nx, *nz, *ntvec, *nrpar, ipar[], *nipar;
     integer insz[], *nin, outsz[], *nout;
     double x[], xd[], z[], tvec[], rpar[];
     double *inptr[], *outptr[], *t;

/* Integer parameters
 *
 * ipar[0]   : device number
 * ipar[1:n] : list of buttons
 *
 */

{
	if (*flag == 1)		/* outputs update */
	{
		int i;


		JSUpdate (&(sci_js[ipar[0]].jsd));

		/* retrieve button states */
		for (i = 0; i < outsz[0]; i++)
			*(outptr[0] + i) =
				JSGetButtonState (&(sci_js[ipar[0]].jsd),
						  ipar[1 + i]);
	}
	else if (*flag == 4)	/* initialize */
	{
		int i, nb;

		/* initialize joystick device */
		if (sci_js_init (ipar[0]) < 0)
		{
			*flag = -3;
			return 0;
		}

		/* number of requested buttons */
		nb = (*nipar - 1);

		/* check for requested buttons */
		for (i = 1; i <= nb; i++)
			if (!JSIsButtonAllocated
			    (&(sci_js[ipar[0]].jsd), ipar[i]))
			{
				sciprint ("JOYSTICK: /dev/js%i: button %i not available.\r\n",
					  ipar[0], ipar[i]);
				*flag = -3;
				return 0;
			}
	}
	else if (*flag == 5)
		sci_js_close (ipar[0]);

	return 0;
}
